/*
 * Copyright (c) 2001,2002 Florian Schulze.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the authors nor the names of the contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * test.c - This file is a simple application for MSDOS/djgpp. It's based
 *          on src/contrrib/ports/win32/test.c.
 *
 */

#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <conio.h>
#include <signal.h>
#include <unistd.h>

#include "lwip/opt.h"
#include "lwip/sys.h"
#include "lwip/timeouts.h"
#include "lwip/debug.h"
#include "lwip/init.h"

#include "lwip/tcpip.h"
#include "lwip/netif.h"
#include "lwip/api.h"

#include "lwip/tcp.h"
#include "lwip/udp.h"
#include "lwip/dns.h"
#include "lwip/dhcp.h"
#include "lwip/stats.h"
#include "lwip/etharp.h"
#include "lwip/etharp.h"
#include "lwip/apps/netbiosns.h"
#include "lwip/apps/sntp.h"
#include "lwip/apps/httpd.h"

#include "netif/ethernet.h" /* ethernet_input() */

#include "./pkt_if.h"

/* applications includes */
#include "apps/snmp/snmp_msg.h"
#include "lwip/apps/snmp_mib2.h"

#include "apps/httpserver/httpserver-netconn.h"
#include "apps/netio/netio.h"
#include "apps/ping/ping.h"
#include "apps/rtp/rtp.h"
#include "apps/chargen/chargen.h"
#include "apps/shell/shell.h"
#include "apps/tcpecho/tcpecho.h"
#include "apps/udpecho/udpecho.h"
#include "apps/socket_examples/socket_examples.h"

#if NO_SYS && 0
  /*
   * We need information about the timer intervals:
   */
  #include "lwip/igmp.h"
#endif

#define MAIN_LOOP_TIMEOUT 2000

/*
 * Defines for my IP-address, gateway and netmask.
 */
#if !LWIP_DHCP
  #define LWIP_PORT_INIT_IPADDR(addr)   IP4_ADDR ((addr), 10, 0, 0, 8)
  #define LWIP_PORT_INIT_GW(addr)       IP4_ADDR ((addr), 10, 0, 0, 1)
  #define LWIP_PORT_INIT_NETMASK(addr)  IP4_ADDR ((addr), 255, 255, 255, 0)
#endif

#if LWIP_SNMP
  /* SNMP trap destination cmd option
   */
  static unsigned char trap_flag = 1;
  static ip_addr_t     trap_addr;

  /* 'non-volatile' SNMP settings
   * @todo: make these truly non-volatile
    */
  u8_t  syscontact_str[255];
  u16_t syscontact_len = 0;
  u8_t  syslocation_str[255];
  u16_t syslocation_len = 0;

  /* enable == 1, disable == 2 */
  u8_t snmpauthentraps_set = 1;
#endif

/* globales variables for netifs */

/* The ethernet interface */
struct netif netif;

#if LWIP_DHCP
  /* dhcp struct for the ethernet netif */
  struct dhcp netif_dhcp;
#endif


/* This function initializes the network interface(s)
 */
static void dos_netif_init (void)
{
  struct netif *iface;

  ip4_addr_t ipaddr, netmask, gw;

  ip4_addr_set_zero (&gw);
  ip4_addr_set_zero (&ipaddr);
  ip4_addr_set_zero (&netmask);

#if LWIP_DHCP
  LWIP_TRACE (0, "Starting lwIP, local interface IP is dhcp-enabled\n");
#else
  LWIP_PORT_INIT_GW (&gw);
  LWIP_PORT_INIT_IPADDR (&ipaddr);
  LWIP_PORT_INIT_NETMASK (&netmask);
  LWIP_TRACE (0, "Starting lwIP, local interface IP is %s\n", ip4addr_ntoa(&ipaddr));
#endif

  iface = netif_add (&netif, &ipaddr, &netmask, &gw, NULL, pkt_init, ethernet_input);
  if (!iface)
  {
    printf ("netif_add() failed\n");
    exit (-1);
  }

  netif_set_default (iface);
  netif_set_up (iface);

#if LWIP_DHCP
  dhcp_set_struct (&netif, &netif_dhcp);
  dhcp_start (&netif);
#endif
}

#if LWIP_DNS_APP && LWIP_DNS
static void dns_found (const char *name, const struct ip_addr *addr, void *arg)
{
  LWIP_UNUSED_ARG (arg);
  LWIP_TRACE (0, "%s: %s\n", name, addr ? ip_ntoa(addr) : "<not found>");
}

static void dns_dorequest (void *arg)
{
  const char *dnsname = "www.3com.com";
  ip_addr_t   dnsresp;

  LWIP_UNUSED_ARG (arg);

  if (dns_gethostbyname(dnsname, &dnsresp, dns_found, 0) == ERR_OK)
    dns_found(dnsname, &dnsresp, 0);
}
#endif

/*
 * This function initializes applications.
 */
static void apps_init (void)
{
#if LWIP_DNS_APP && LWIP_DNS
  /* wait until the netif is up (for dhcp, autoip or ppp) */
  sys_timeout (1000, dns_dorequest, NULL);
#endif

#if LWIP_CHARGEN_APP && LWIP_SOCKET
  chargen_init();
#endif

#if LWIP_PING_APP && LWIP_RAW && LWIP_ICMP
  ping_init();
#endif

#if LWIP_NETBIOS_APP && LWIP_UDP
  netbiosns_init();
#endif

#if LWIP_HTTPD_APP && LWIP_TCP
  httpd_init();
#endif

#if LWIP_NETIO_APP && LWIP_TCP
  netio_init();
#endif

#if LWIP_RTP_APP && LWIP_SOCKET && LWIP_IGMP
  rtp_init();
#endif

#if LWIP_SNTP_APP && LWIP_SOCKET
  sntp_init();
#endif

#if LWIP_SHELL_APP && LWIP_NETCONN
  shell_init();
#endif

#if LWIP_TCPECHO_APP
  #if LWIP_NETCONN && defined(LWIP_TCPECHO_APP_NETCONN)
    tcpecho_init();
  #endif
#endif

#if LWIP_SNMP
  snmp_trap_dst_ip_set (0, &trap_addr);
  snmp_trap_dst_enable (0, trap_flag);
  snmp_mib2_set_syscontact (syscontact_str, &syscontact_len, sizeof(syscontact_str));
  snmp_mib2_set_syslocation (syslocation_str, &syslocation_len, sizeof(syslocation_str));
  snmp_set_auth_traps_enabled (snmpauthentraps_set);
  snmp_init();
#endif

#if LWIP_UDPECHO_APP && LWIP_NETCONN
  udpecho_init();
#endif

#if LWIP_SOCKET_EXAMPLES_APP && LWIP_SOCKET
  socket_examples_init();
#endif

#ifdef LWIP_APP_INIT
  LWIP_APP_INIT();
#endif
}

/* All jobs are done here
 */
static void main_loop (void)
{
  /* handle timers (already done in tcpip.c when NO_SYS=0) */
  sys_check_timeouts();

  /* check for packets */
  pkt_poll (&netif);

  /* When pkt_poll comes back, there are no packets, so sleep to
   * prevent 100% CPU load. Don't do this in an embedded system since it
   * increases latency! Don't delay too much (or not at all). Otherwise the
   * packet-driver will surely loose packets (the PktRxDropped() count will
   * rise).
   */
#if 0
  delay (5);  /* since sys_msleep() is a NOP when NO_SYS=1, we use delay() */
#endif

#if ENABLE_LOOPBACK && !LWIP_NETIF_LOOPBACK_MULTITHREADING
  /* check for loopback packets on all netifs */
  netif_poll_all();
#endif
}

static int exit_app (int rc)
{
  pkt_exit (&netif);
  fflush (stdout);
  return (rc);
}

static void main_loop_poller (void *arg)
{
#if 1
  putchar ('.');
  fflush (stdout);
  (void) arg;
#else
  printf ("%s(%s): %lu\n", __FUNCTION__, (const char*)arg, sys_now());
#endif
  sys_timeout (MAIN_LOOP_TIMEOUT, main_loop_poller, "hello");
}

int main (int argc, char **argv)
{
#if LWIP_SNMP
  const char *e = getenv ("LWIP_TRAP");

  if (e)
  {
    char ip_str[16];

    snmpauthentraps_set = 1;
    ipaddr_aton (e, &trap_addr);
    strncpy (ip_str, ipaddr_ntoa(&trap_addr),sizeof(ip_str));
    LWIP_TRACE (0, "SNMP trap destination %s\n", ip_str);
  }

  setvbuf (stdout, NULL,_IONBF, 0);
#endif

  /* initialize everything:
   */
  sys_init();
  lwip_init();
  dos_netif_init();
  apps_init();

  sys_timeout (MAIN_LOOP_TIMEOUT, main_loop_poller, "hello");

  while (1)
  {
    main_loop();
    if (kbhit() && getch() == 'q')
       break;
  }
  return exit_app(0);
}
